import 'package:flutter/material.dart';

import 'scale_scheme.dart';
import 'scale_theme.dart';

class ScaleInputDecoratorTheme extends InputDecorationTheme {
  ScaleInputDecoratorTheme(
      this._scaleScheme, ScaleConfig scaleConfig, this._textTheme)
      : super(
            border: OutlineInputBorder(
                borderSide: BorderSide(color: _scaleScheme.primaryScale.border),
                borderRadius:
                    BorderRadius.circular(8 * scaleConfig.borderRadiusScale)),
            contentPadding: const EdgeInsets.all(8),
            labelStyle: TextStyle(
                color: _scaleScheme.primaryScale.subtleText.withAlpha(127)),
            floatingLabelStyle:
                TextStyle(color: _scaleScheme.primaryScale.subtleText),
            focusedBorder: OutlineInputBorder(
                borderSide: BorderSide(
                    color: _scaleScheme.primaryScale.hoverBorder, width: 2),
                borderRadius:
                    BorderRadius.circular(8 * scaleConfig.borderRadiusScale)));

  final ScaleScheme _scaleScheme;
  final TextTheme _textTheme;

  @override
  TextStyle? get hintStyle => WidgetStateTextStyle.resolveWith((states) {
        if (states.contains(WidgetState.disabled)) {
          return TextStyle(color: _scaleScheme.grayScale.border);
        }
        return TextStyle(color: _scaleScheme.primaryScale.border);
      });

  @override
  Color? get fillColor => WidgetStateColor.resolveWith((states) {
        if (states.contains(WidgetState.disabled)) {
          return _scaleScheme.grayScale.primary.withAlpha(10);
        }
        return _scaleScheme.primaryScale.primary.withAlpha(10);
      });

  @override
  BorderSide? get activeIndicatorBorder =>
      WidgetStateBorderSide.resolveWith((states) {
        if (states.contains(WidgetState.disabled)) {
          return BorderSide(
              color: _scaleScheme.grayScale.border.withAlpha(127));
        }
        if (states.contains(WidgetState.error)) {
          if (states.contains(WidgetState.hovered)) {
            return BorderSide(color: _scaleScheme.errorScale.hoverBorder);
          }
          if (states.contains(WidgetState.focused)) {
            return BorderSide(color: _scaleScheme.errorScale.border, width: 2);
          }
          return BorderSide(color: _scaleScheme.errorScale.subtleBorder);
        }
        if (states.contains(WidgetState.hovered)) {
          return BorderSide(color: _scaleScheme.secondaryScale.hoverBorder);
        }
        if (states.contains(WidgetState.focused)) {
          return BorderSide(
              color: _scaleScheme.secondaryScale.border, width: 2);
        }
        return BorderSide(color: _scaleScheme.secondaryScale.subtleBorder);
      });

  @override
  BorderSide? get outlineBorder => WidgetStateBorderSide.resolveWith((states) {
        if (states.contains(WidgetState.disabled)) {
          return BorderSide(
              color: _scaleScheme.grayScale.border.withAlpha(127));
        }
        if (states.contains(WidgetState.error)) {
          if (states.contains(WidgetState.hovered)) {
            return BorderSide(color: _scaleScheme.errorScale.hoverBorder);
          }
          if (states.contains(WidgetState.focused)) {
            return BorderSide(color: _scaleScheme.errorScale.border, width: 2);
          }
          return BorderSide(color: _scaleScheme.errorScale.subtleBorder);
        }
        if (states.contains(WidgetState.hovered)) {
          return BorderSide(color: _scaleScheme.primaryScale.hoverBorder);
        }
        if (states.contains(WidgetState.focused)) {
          return BorderSide(color: _scaleScheme.primaryScale.border, width: 2);
        }
        return BorderSide(color: _scaleScheme.primaryScale.subtleBorder);
      });

  @override
  Color? get iconColor => _scaleScheme.primaryScale.primary;

  @override
  Color? get prefixIconColor => WidgetStateColor.resolveWith((states) {
        if (states.contains(WidgetState.disabled)) {
          return _scaleScheme.primaryScale.primary.withAlpha(127);
        }
        if (states.contains(WidgetState.error)) {
          return _scaleScheme.errorScale.primary;
        }
        return _scaleScheme.primaryScale.primary;
      });

  @override
  Color? get suffixIconColor => WidgetStateColor.resolveWith((states) {
        if (states.contains(WidgetState.disabled)) {
          return _scaleScheme.primaryScale.primary.withAlpha(127);
        }
        if (states.contains(WidgetState.error)) {
          return _scaleScheme.errorScale.primary;
        }
        return _scaleScheme.primaryScale.primary;
      });

  @override
  TextStyle? get labelStyle => WidgetStateTextStyle.resolveWith((states) {
        final textStyle = _textTheme.bodyLarge ?? const TextStyle();
        if (states.contains(WidgetState.disabled)) {
          return textStyle.copyWith(
              color: _scaleScheme.grayScale.border.withAlpha(127));
        }
        if (states.contains(WidgetState.error)) {
          if (states.contains(WidgetState.hovered)) {
            return textStyle.copyWith(
                color: _scaleScheme.errorScale.hoverBorder);
          }
          if (states.contains(WidgetState.focused)) {
            return textStyle.copyWith(
                color: _scaleScheme.errorScale.hoverBorder);
          }
          return textStyle.copyWith(
              color: _scaleScheme.errorScale.subtleBorder);
        }
        if (states.contains(WidgetState.hovered)) {
          return textStyle.copyWith(
              color: _scaleScheme.primaryScale.hoverBorder);
        }
        if (states.contains(WidgetState.focused)) {
          return textStyle.copyWith(
              color: _scaleScheme.primaryScale.hoverBorder);
        }
        return textStyle.copyWith(color: _scaleScheme.primaryScale.border);
      });

  @override
  TextStyle? get floatingLabelStyle => labelStyle;

  @override
  TextStyle? get helperStyle => WidgetStateTextStyle.resolveWith((states) {
        final textStyle = _textTheme.bodySmall ?? const TextStyle();
        if (states.contains(WidgetState.disabled)) {
          return textStyle.copyWith(
              color: _scaleScheme.grayScale.border.withAlpha(127));
        }
        return textStyle.copyWith(
            color: _scaleScheme.secondaryScale.border.withAlpha(127));
      });

  @override
  TextStyle? get errorStyle => WidgetStateTextStyle.resolveWith((states) {
        final textStyle = _textTheme.bodySmall ?? const TextStyle();
        return textStyle.copyWith(color: _scaleScheme.errorScale.primary);
      });
}

extension ScaleInputDecoratorThemeExt on ScaleTheme {
  ScaleInputDecoratorTheme inputDecoratorTheme() =>
      ScaleInputDecoratorTheme(scheme, config, textTheme);
}
